from spykeutils.plugin import analysis_plugin, gui_data
from spykeutils.plot.dialog import PlotDialog
from spykeutils import SpykeException
from spykeutils.conversions import analog_signal_array_to_analog_signals

from collections import OrderedDict

import scipy as sp
import quantities as pq
import matplotlib.pyplot as plt
import neo

from guiqwt.builder import make
from guiqwt.baseplot import BasePlot
from guiqwt.plot import BaseCurveWidget

class STAPlugin(analysis_plugin.AnalysisPlugin):
    left_offset = gui_data.IntItem('Left offset', default=100)
    right_offset = gui_data.IntItem('Right offset', default=100)
    
    def get_name(self):
        return 'Spike Triggered Average'

    def start(self, current, selections):
        current.progress.begin()
        signals_by_channel = current.analog_signals_by_channel_and_segment()
        trains_by_unit = current.spike_trains_by_unit_and_segment()
        
        if not trains_by_unit:
            raise SpykeException('Select at least one spike train')
        if not signals_by_channel:
            raise SpykeException('Select at least one signal')

        sta = OrderedDict()
        
        current.progress.set_ticks(
            len(signals_by_channel) * len(trains_by_unit) *
            len(current.segments()))
        current.progress.set_status('Calculating')
        
        # Calculate STA
        for u, trains in trains_by_unit.iteritems():
            sta[u] = OrderedDict()
            for c, signals in signals_by_channel.iteritems():
                sts = sp.zeros(self.left_offset + self.right_offset) * pq.uV
                num = 0
                for seg, train in trains.iteritems():
                    try:
                        sig = signals[seg]
                        
                        n = self.calc_sta(train, sig, sts)
                        num += n
                    except KeyError:
                        pass
                    finally:
                        current.progress.step()
                if num > 0:
                    sts /= num
                sta[u][c] = sts

        # Plot
        win_title = 'Spike Triggered Average'
        win = PlotDialog(toolbar=True, wintitle=win_title, min_plot_width=150,
                         min_plot_height=100)
                         
        x = sp.arange(-self.left_offset, self.right_offset)
        pid = 0
        i = 0
        for u, inner in sta.iteritems():
            j = 0
            for c, values in inner.iteritems():
                pW = BaseCurveWidget(win)
                plot = pW.plot
                plot.set_antialiasing(True)
                plot.add_item(make.curve(x, values))
                
                if j == len(inner) - 1: # Name and axis description for lowest plots
                    if u is not None:
                        plot.set_axis_title(BasePlot.X_BOTTOM, 'Sample - %s' % u.name)
                    else:
                        plot.set_axis_title(BasePlot.X_BOTTOM, 'Sample')
                        
                if i == 0: # Name and axis description for plots on left column
                    if c is not None:
                        plot.set_axis_title(BasePlot.Y_LEFT, 'STA - %s' % c.name)
                    else:
                        plot.set_axis_title(BasePlot.Y_LEFT, 'STA')
                    plot.set_axis_unit(BasePlot.Y_LEFT, values.dimensionality.string)
                
                win.add_plot_widget(pW, pid, j, i)
                pid += 1
                j += 1
            i += 1
        win.add_custom_curve_tools()
        current.progress.done()
        win.show()
        
        if pid > 1:
            win.add_x_synchronization_option(True, range(pid))
            win.add_y_synchronization_option(False, range(pid))
    
        return win
        
    def calc_sta(self, train, sig, sts):
        """ Calculate spike triggered sum for a single spike train and signal.
        Returns number of spikes used. """
        num = 0
        for s in train:
            i = sig.times.searchsorted(s.rescale(sig.times.units), 'right') - 1
            if i < self.left_offset or i > sig.shape[0] - self.right_offset:
                continue
            sts += sig[i - self.left_offset: i + self.right_offset]
            num += 1
    
        return num